#include "Status.h"
#include "Vram.h"
#include "VVram.h"
#include "Print.h"
#include "Chars.h"
#include "Sprite.h"

constexpr byte Char_Remain = Char_Fighter;

byte[WindowWidth * 2]  Status;

static ptr<byte> PrintS(ptr<byte> pStatus, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pStatus = PrintC(pStatus, c);
        ++p;
    }
    return pStatus;
}

void PrintStatus()
{
    PrintS(Status + WindowWidth * 0 + 0, "SCORE");
    PrintS(Status + WindowWidth * 0 + 9, "HI-SCORE");
    PrintS(Status + WindowWidth * 0 + 18, "STAGE");
    PrintByteNumber2(Status + WindowWidth * 1 + 21, CurrentStage + 1);
    PrintRemain();
    PrintScore();
}

void PrintScore()
{
    ptr<byte> pStatus;
    
    pStatus = PrintNumber5(Status + WindowWidth * 1 + 0, Score);
    PrintC(pStatus, '0');

    pStatus = PrintNumber5(Status + WindowWidth * 1 + 11, HiScore);
    PrintC(pStatus, '0');
}

void PrintRemain()
{
    ptr<byte> pStatus;
    pStatus = Status + WindowWidth * 0 + 24;
    if (RemainCount > 1) {
        byte i;
        i = RemainCount - 1;
        if (i > 2) {
            pStatus = VPut2C(pStatus, Char_Remain);
            pStatus = VErase2(pStatus);
            pStatus += WindowWidth - 2;
            pStatus = PrintC(pStatus, i + '0');
            pStatus += 1 - WindowWidth;
        }
        else {
            do {
                pStatus = VPut2C(pStatus, Char_Remain);
                --i;
            } while (i > 0);
        }
    }
    if (RemainCount < 3) {
        pStatus = VErase2(pStatus);
    }
}


void PrintGameOver()
{
    constexpr byte Left = (WindowWidth - 8) / 2;
    PrintS(Vram + VramRowSize * (VramHeight / 2) + Left * VramStep, "GAME OVER");
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    // UpdateSprites();
    PrintStatus();
    StatusToVram();
    {
        static const byte[] TitleBytes = {
            //	G
            0x1e, 0x2c, 0x23, 0x29, 
            0x2a, 0x21, 0x28, 0x28, 
            0x22, 0x29, 0x1e, 0x2d, 
            0x1e, 0x22, 0x23, 0x23, 
            //	U
            0x2a, 0x21, 0x1e, 0x2d, 
            0x2a, 0x21, 0x1e, 0x2d, 
            0x2a, 0x21, 0x1e, 0x2d, 
            0x1e, 0x23, 0x23, 0x1f, 
            //	N
            0x2a, 0x29, 0x1e, 0x2d, 
            0x2a, 0x2d, 0x29, 0x2d, 
            0x2a, 0x21, 0x2b, 0x2d, 
            0x22, 0x1f, 0x1e, 0x23, 
            //	T
            0x22, 0x2b, 0x25, 0x1f, 
            0x1e, 0x2a, 0x21, 0x1e, 
            0x1e, 0x2a, 0x21, 0x1e, 
            0x1e, 0x22, 0x1f, 0x1e, 
            //	U
            0x2a, 0x21, 0x1e, 0x2d, 
            0x2a, 0x21, 0x1e, 0x2d, 
            0x2a, 0x21, 0x1e, 0x2d, 
            0x1e, 0x23, 0x23, 0x1f, 
            //	S
            0x26, 0x25, 0x23, 0x29, 
            0x22, 0x29, 0x28, 0x20, 
            0x26, 0x20, 0x1e, 0x2d, 
            0x1e, 0x23, 0x23, 0x1f, 
        };
        constexpr byte LogoLength = 6;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + 2 + VramRowSize * 7 + (WindowWidth - LogoLength * 4)  * VramStep / 2;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (4) {
                repeat (4) {
                    *pVram = *p; ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 4;
        }
    }
    PrintS(Vram + 2 + VramRowSize * 19 + (WindowWidth - 19) * VramStep / 2, "PUSH TRIGGER BUTTON");
    PrintS(Vram + 2 + VramRowSize * 23 + (WindowWidth - 12) * VramStep, "INUFUTO 2022");
    // {
    //     byte c;
    //     ptr<byte> pVram;
    //     pVram = Vram+2;
    //     c = 0;
    //     repeat (0) {
    //         *pVram = c; ++pVram;
    //         ++c;
    //         if ((c & 15) == 0) {
    //             pVram += VramRowSize - 16;
    //         }
    //     }
    // }
}


void AddScore(word pts)
{
    Score += pts;
    if (Score > HiScore) {
        HiScore = Score;
    }
    PrintScore();
}
